from enum import Enum


class CurrenciesIDs(Enum):
    IRT = 2
    VTHO = 92
    ALGO = 93
    ATLAS = 148
    BTC = 1
    ETH = 3
    USDT = 4
    BNB = 10
    ADA = 9
    SOL = 51
    XRP = 5
    BCH = 7
    DOT = 8
    USDC = 27
    XLM = 11
    BABYDOGE = 57
    LTC = 6
    XMR = 12
    UNI = 22
    EOS = 13
    LINK = 26
    XEM = 14
    TRX = 15
    XTZ = 16
    THETA = 17
    VET = 18
    ATOM = 19
    NEO = 20
    DASH = 21
    DOGE = 33
    GRT = 37
    FIL = 23
    AAVE = 24
    BUSD = 31
    WAVES = 25
    ZEC = 29
    YFI = 28
    DAI = 34
    GALA = 127
    ETC = 30
    SHIB = 40
    COMP = 32
    SUSHI = 35
    MKR = 36
    BTT = 38
    IOTA = 39
    ONE = 50
    MANA = 54
    CELR = 47
    CHZ = 46
    EGLD = 52
    TWT = 45
    ZIL = 53
    CRV = 144
    MATIC = 44
    CAKE = 43
    ANKR = 55
    FTM = 42
    WIN = 41
    TOMO = 48
    SAFEMOON = 49
    SAND = 62
    MIR = 64
    ONEINCH = 63
    AXS = 67
    AVAX = 68
    ALICE = 66
    ENJ = 65
    REEF = 59
    TLM = 61
    HOT = 60
    KISHU = 56
    FEG = 58
    FORTH = 69
    QNT = 70
    IOTX = 139
    LINA = 71
    DODO = 72
    CTK = 89
    DENT = 88
    CHR = 87
    BEL = 86
    EPS = 85
    C98 = 84
    BAND = 83
    INJ = 82
    XVS = 81
    SXP = 80
    ONT = 79
    BAT = 78
    SNX = 77
    NEAR = 76
    SKL = 75
    LUNA = 90
    MASK = 74
    DYDX = 91
    COTI = 73
    SLP = 96
    FTT = 95
    BNT = 143
    KSM = 94
    AUDIO = 142
    ELON = 128
    MDX = 137
    POLS = 141
    BAKE = 140
    SFM = 149
    BTTC = 150
    BLOK = 147
    CITY = 146
    VRA = 129
    PORTO = 138
    JOE = 145
    PSG = 136
    LAZIO = 135
    CQT = 131
    ATM = 133
    JUV = 134
    ASR = 132
    FLOKI = 130


class MarketsIDs(Enum):
    BTC_IRT = 1
    BTC_USDT = 2
    ETH_USDT = 3
    ETH_IRT = 4
    USDT_IRT = 5
    ADA_USDT = 15
    BNB_IRT = 16
    LINK_USDT = 19
    SOL_USDT = 109
    ADA_IRT = 14
    TRX_USDT = 31
    SOL_IRT = 108
    XRP_IRT = 6
    LTC_USDT = 9
    DOT_USDT = 13
    XRP_USDT = 7
    DOT_IRT = 12
    DOGE_USDT = 63
    DOGE_IRT = 62
    UNI_USDT = 45
    SHIB_IRT = 78
    BNB_USDT = 17
    LUNA_IRT = 178
    USDC_IRT = 20
    XLM_USDT = 23
    AVAX_IRT = 110
    LTC_IRT = 8
    USDC_USDT = 21
    XMR_IRT = 24
    XMR_USDT = 25
    UNI_IRT = 44
    LINK_IRT = 18
    XEM_IRT = 28
    XEM_USDT = 29
    BCH_IRT = 10
    BCH_USDT = 11
    BUSD_IRT = 50
    THETA_USDT = 35
    MATIC_IRT = 86
    MATIC_USDT = 87
    ALGO_IRT = 184
    VET_IRT = 36
    XLM_IRT = 22
    VET_USDT = 37
    AXS_IRT = 113
    ATOM_USDT = 39
    NEO_IRT = 40
    NEO_USDT = 41
    FIL_IRT = 46
    TRX_IRT = 30
    DASH_IRT = 42
    DASH_USDT = 43
    ATOM_IRT = 38
    ETC_IRT = 58
    THETA_IRT = 34
    FIL_USDT = 47
    FTT_IRT = 186
    FTM_IRT = 82
    FTM_USDT = 83
    DAI_IRT = 66
    DAI_USDT = 67
    BUSD_USDT = 51
    EGLD_IRT = 104
    WAVES_IRT = 52
    WAVES_USDT = 53
    NEAR_IRT = 171
    GRT_USDT = 68
    GRT_IRT = 69
    XTZ_IRT = 32
    XTZ_USDT = 33
    ETC_USDT = 59
    EOS_IRT = 26
    EOS_USDT = 27
    GALA_IRT = 221
    GALA_USDT = 222
    CAKE_IRT = 84
    CAKE_USDT = 85
    MANA_IRT = 106
    MANA_USDT = 107
    AAVE_IRT = 48
    AAVE_USDT = 49
    IOTA_USDT = 74
    IOTA_IRT = 75
    SHIB_USDT = 79
    KSM_IRT = 190
    KSM_USDT = 191
    QNT_USDT = 138
    QNT_IRT = 139
    CRV_IRT = 251
    CRV_USDT = 252
    ONE_IRT = 98
    ONE_USDT = 99
    MKR_USDT = 72
    MKR_IRT = 73
    CHZ_IRT = 90
    CHZ_USDT = 91
    EGLD_USDT = 105
    BTT_USDT = 76
    BTT_IRT = 77
    AVAX_USDT = 111
    HOT_USDT = 124
    HOT_IRT = 125
    AXS_USDT = 112
    ENJ_USDT = 130
    ENJ_IRT = 131
    ZEC_IRT = 54
    ZEC_USDT = 55
    COMP_IRT = 60
    COMP_USDT = 61
    SAND_USDT = 116
    SAND_IRT = 117
    BAT_USDT = 166
    BAT_IRT = 167
    IOTX_IRT = 245
    IOTX_USDT = 246
    SUSHI_USDT = 70
    SUSHI_IRT = 71
    ZIL_IRT = 102
    ZIL_USDT = 103
    YFI_IRT = 56
    YFI_USDT = 57
    SNX_USDT = 168
    SNX_IRT = 169
    NEAR_USDT = 170
    LUNA_USDT = 179
    ALGO_USDT = 185
    BNT_IRT = 253
    BNT_USDT = 254
    ANKR_IRT = 100
    ANKR_USDT = 101
    FTT_USDT = 187
    ONT_USDT = 164
    ONT_IRT = 165
    DYDX_IRT = 180
    DYDX_USDT = 181
    SKL_USDT = 172
    SKL_IRT = 173
    ONEINCH_USDT = 132
    ONEINCH_IRT = 133
    ELON_IRT = 223
    ELON_USDT = 224
    CELR_IRT = 92
    CELR_USDT = 93
    WIN_IRT = 80
    WIN_USDT = 81
    DENT_USDT = 146
    DENT_IRT = 147
    AUDIO_IRT = 255
    AUDIO_USDT = 256
    ALICE_USDT = 114
    ALICE_IRT = 115
    INJ_USDT = 158
    INJ_IRT = 159
    CHR_USDT = 148
    CHR_IRT = 149
    SXP_USDT = 162
    SXP_IRT = 163
    REEF_USDT = 128
    REEF_IRT = 129
    MDX_IRT = 241
    MDX_USDT = 242
    COTI_USDT = 176
    COTI_IRT = 177
    VTHO_IRT = 182
    VTHO_USDT = 183
    POLS_IRT = 249
    POLS_USDT = 250
    BAKE_IRT = 247
    BAKE_USDT = 248
    FLOKI_IRT = 227
    FLOKI_USDT = 228
    BLOK_IRT = 261
    BLOK_USDT = 262
    BTTC_USDT = 267
    BTTC_IRT = 268
    SAFEMOON_USDT = 96
    SAFEMOON_IRT = 97
    C98_USDT = 154
    C98_IRT = 155
    MASK_USDT = 174
    MASK_IRT = 175
    TWT_IRT = 88
    TWT_USDT = 89
    TLM_IRT = 126
    TLM_USDT = 127
    XVS_USDT = 160
    XVS_IRT = 161
    SFM_USDT = 266
    TOMO_IRT = 94
    TOMO_USDT = 95
    MIR_USDT = 134
    MIR_IRT = 135
    BAND_USDT = 156
    BAND_IRT = 157
    VRA_IRT = 225
    VRA_USDT = 226
    LINA_USDT = 140
    LINA_IRT = 141
    EPS_USDT = 152
    EPS_IRT = 153
    SLP_IRT = 188
    SLP_USDT = 189
    CTK_USDT = 144
    CTK_IRT = 145
    FORTH_USDT = 136
    FORTH_IRT = 137
    DODO_USDT = 142
    DODO_IRT = 143
    BEL_USDT = 150
    BEL_IRT = 151
    PORTO_IRT = 243
    PORTO_USDT = 244
    PSG_IRT = 239
    PSG_USDT = 240
    LAZIO_USDT = 237
    LAZIO_IRT = 238
    CQT_IRT = 229
    CQT_USDT = 230
    ATM_IRT = 234
    ATM_USDT = 233
    JUV_IRT = 235
    JUV_USDT = 236
    ASR_IRT = 231
    ASR_USDT = 232
    BABYDOGE_USDT = 120
    BABYDOGE_IRT = 121
    KISHU_USDT = 118
    KISHU_IRT = 119
    FEG_USDT = 122
    FEG_IRT = 123
    SFM_IRT = 265
    CRO_IRT = 312
